//*****************************************
// AT mega328P-AU(32pin TQFP) four IN-4 & ZM1020 clock Program
// CPU Clock:8MHz
//   iq[Yf[^@WȂBIV[^(8MHz)ɂj
//
// File Name:ad_conv.c
//
//  2017/09/13 New TAB4 Taishi
//  2017/12/13 Ver 2.00 Taishi
//  2018/10/25 Modify for Fifteen IN-17 Clock Taishi
//  2019/12/30 Modify for fourteen IN-172 Clock Taishi
//  2020/09/17 Modify for six IN-16 clock Taishi
//  2020/09/24 Modify for four IN-8 clock Taishi
//  2022/02/26 Modify for four IN-4 & ZM1020 clock Taishi
//*****************************************
#include				<avr/io.h>

#include	"four_in4_zm1020_clock.h"
#include	"ad_conv.h"

#if	AD_NO == 6
static	int				ad6_data[N_of_AD_data];
int						ad6_ave;
static	unsigned char	ad6_count;
// *******************************************************
// ************************* A/D 6 ***********************
// *******************************************************

//*****************************************
// A/D 6 Initial
// @荞ݕs̏ԂŌĂ΂Ƃ
//  ADC6: High Volt Monitor Max. about 0.55V (512 = 200V)
//*****************************************
void	ad6_init(void)
{
	unsigned char	i;

	ad6_count = 0;
	for(i = 0; i < N_of_AD_data; i++){
		ad6_data[i] = 0;
	}
	ad6_ave = 0;


	PRR &= ~( 1 << PRADC );																	// A/D ғ

	ADMUX = ( (0 << REFS1) | (1 << REFS0) | (0 << ADLAR) | (0 << MUX3) | (1 << MUX2) | (1 << MUX1) | (0 << MUX0) );	// ubbdgp, Select ADC6
	ADCSRA = ( (1 << ADEN) | (1 << ADSC) | (1 << ADPS2) | (1 << ADPS1) | (1 << ADPS0) );				// A/D, A/DϊJn, CK/128=8MHz/128=62.5kHz(16uS) --> 208uS

}
//*****************************************
// A/D 6 close
//*****************************************
void	ad6_close(void)
{
	ADCSRA = 0;																				// A/Ds
}
//*****************************************
// A/D 6 Set A/D data to Data Buffer
//  ϊIĂȂ́@Ȃ
//*****************************************
void	ad6_set_data(void)
{
	if( (ADCSRA & (1 << ADIF)) != 0){								// `^cϊ
		ADCSRA |= (1 << ADIF);										// `^cϊtO폜
		ad6_data[ad6_count] = (ADCW & 0x3ff);
		ad6_count++;
		ad6_count &= (N_of_AD_data - 1);

		ADCSRA |= (1 << ADSC);										// A/DϊJn
	}
}
//*****************************************
// A/D 6 Get A/D data
//  obt@ɓĂf[^𕽋ςĎ擾
//  擾f[^ ad0_ave ɓ
//*****************************************
void	ad6_get_data(void)
{
	unsigned char	i;
	unsigned int	sum;

	sum = 0;
	for(i = 0; i < N_of_AD_data; i++){						// obt@Sđ
		sum += ad6_data[i];
	}

	sum /= N_of_AD_data;									// f[^Ŋĕϒlɂ

	ad6_ave = sum;
}

#else
 #if	AD_NO == 5
static	int				ad5_data[N_of_AD_data];
int						ad5_ave;
static	unsigned char	ad5_count;
// *******************************************************
// ************************* A/D 5 ***********************
// *******************************************************

//*****************************************
// A/D 5 Initial
// @荞ݕs̏ԂŌĂ΂Ƃ
//  PA5(ADC0): High Volt Monitor Max. about 0.55V (512 = 200V)
//*****************************************
void	ad5_init(void)
{
	unsigned char	i;

	ad5_count = 0;
	for(i = 0; i < N_of_AD_data; i++){
		ad5_data[i] = 0;
	}
	ad5_ave = 0;

	DDRC &= ~(1 << DDC5);																	// Port-C Bit5 : Input

	PRR &= ~( 1 << PRADC );																	// A/D ғ
	DIDR0 |= (1 << ADC5D);																	// AiO͐ݒ AD5

	ADMUX = ( (0 << REFS1) | (1 << REFS0) | (0 << MUX3) | (1 << MUX2) | (0 << MUX1) | (1 << MUX0) );		// ubbdgp, Select ADC5
	ADCSRA = ( (1 << ADEN) | (1 << ADSC) | (1 << ADPS2) | (1 << ADPS1) | (1 << ADPS0) );	// A/D, A/DϊJn, CK/128=8MHz/128=62.5kHz(16uS) --> 208uS

}
//*****************************************
// A/D 5 close
//*****************************************
void	ad5_close(void)
{
	ADCSRA = 0;																				// A/Ds
}
//*****************************************
// A/D 5 Set A/D data to Data Buffer
//  ϊIĂȂ́@Ȃ
//*****************************************
void	ad5_set_data(void)
{
	if( (ADCSRA & (1 << ADIF)) != 0){								// `^cϊ
		ADCSRA |= (1 << ADIF);										// `^cϊtO폜
		ad5_data[ad5_count] = ADCW;
		ad5_count++;
		ad5_count &= (N_of_AD_data - 1);

		ADCSRA |= (1 << ADSC);										// A/DϊJn
	}
}
//*****************************************
// A/D 5 Get A/D data
//  obt@ɓĂf[^𕽋ςĎ擾
//  擾f[^ ad5_ave ɓ
//*****************************************
void	ad5_get_data(void)
{
	unsigned char	i;
	unsigned int	sum;

	sum = 0;
	for(i = 0; i < N_of_AD_data; i++){						// obt@Sđ
		sum += ad5_data[i];
	}

	sum /= N_of_AD_data;									// f[^Ŋĕϒlɂ

	ad5_ave = sum;
}
 #else
static	int				ad0_data[N_of_AD_data];
int						ad0_ave;
static	unsigned char	ad0_count;
// *******************************************************
// ************************* A/D 0 ***********************
// *******************************************************

//*****************************************
// A/D 0 Initial
// @荞ݕs̏ԂŌĂ΂Ƃ
//  PA0(ADC0): High Volt Monitor Max. about 0.55V (512 = 200V)
//*****************************************
void	ad0_init(void)
{
	unsigned char	i;

	ad0_count = 0;
	for(i = 0; i < N_of_AD_data; i++){
		ad0_data[i] = 0;
	}
	ad0_ave = 0;

	DDRC &= ~(1 << DDC0);																	// Port-C Bit0 : Input

	PRR &= ~( 1 << PRADC );																	// A/D ғ
	DIDR0 |= (1 << ADC0D);																	// AiO͐ݒ AD0

	ADMUX = ( (1 << REFS1) | (0 << REFS0) | (0 << MUX3) | (0 << MUX2) | (0 << MUX1) | (0 << MUX0) );		// PDPudgp, Select ADC0
	ADCSRA = ( (1 << ADEN) | (1 << ADSC) | (1 << ADPS2) | (1 << ADPS1) | (1 << ADPS0) );	// A/D, A/DϊJn, CK/128=8MHz/128=62.5kHz(16uS) --> 208uS

}
//*****************************************
// A/D 0 close
//*****************************************
void	ad0_close(void)
{
	ADCSRA = 0;																				// A/Ds
}
//*****************************************
// A/D 0 Set A/D data to Data Buffer
//  ϊIĂȂ́@Ȃ
//*****************************************
void	ad0_set_data(void)
{
	if( (ADCSRA & (1 << ADIF)) != 0){								// `^cϊ
		ADCSRA |= (1 << ADIF);										// `^cϊtO폜
		ad0_data[ad0_count] = ADCW;
		ad0_count++;
		ad0_count &= (N_of_AD_data - 1);

		ADCSRA |= (1 << ADSC);										// A/DϊJn
	}
}
//*****************************************
// A/D 0 Get A/D data
//  obt@ɓĂf[^𕽋ςĎ擾
//  擾f[^ ad0_ave ɓ
//*****************************************
void	ad0_get_data(void)
{
	unsigned char	i;
	unsigned int	sum;

	sum = 0;
	for(i = 0; i < N_of_AD_data; i++){						// obt@Sđ
		sum += ad0_data[i];
	}

	sum /= N_of_AD_data;									// f[^Ŋĕϒlɂ

	ad0_ave = sum;
}
 #endif
#endif
