//*****************************************
// AT mega328P-AU(32pin TQFP) four IN-4 & ZM1020 clock Program
// CPU Clock:8MHz
//   iq[Yf[^@UWlgɂj
// File Name:rtc_ds3234.c
//
//  PB2(SS)  : SPI-SS
//  PB3(MOSI): SPI-MOSI(Mega328:out, DS3234:in)
//  PB4(MISO): SPI-MISO(Mega328:in,  DS3234:out)
//  PB5(SCT) : SPI-SCK
//
//  PD2(INT0): INT0(interrupt from DS3234)
//
//  C:\WinAVR-20100110\doc\avr-libc\avr-libc-user-manual.pdf ɊeCN[ht@C̐
//
//  2017/09/13 New TAB4 Taishi
//  2017/12/13 Ver 2.00 Taishi
//  2018/10/25 Modify for Fifteen IN-17 Clock Taishi
//  2019/12/30 Modify for fourteen IN-172 Clock Taishi
//  2020/09/17 Modify for six IN-16 clock Taishi
//  2020/09/24 Modify for four IN-8 clock Taishi
//  2022/02/26 Modify for four IN-4 & ZM1020 clock Taishi
//*****************************************

#include	<avr/interrupt.h>
#include	<avr/io.h>
#include	<avr/pgmspace.h>

#include	"four_in4_zm1020_clock.h"
#include	"mega_Spi.h"
#include	"timer.h"
#include	"port_bit.h"

#include	"rtc_ds3234.h"


unsigned char	gRTC_sec;				// qsb b̃f[^ (BCD 0--59)
unsigned char	gRTC_min;				// qsb ̃f[^ (BCD 0--59)
unsigned char	gRTC_hou;				// qsb ̃f[^ (BCD 0--23j
unsigned char	gRTC_wday;				// qsb j̃f[^ (BCD 1--7)
unsigned char	gRTC_day;				// qsb ̃f[^ (BCD 1--31)
unsigned char	gRTC_mon;				// qsb ̃f[^ (BCD 1--12j
unsigned char	gRTC_year;				// qsb Ñf[^ (BCD 0--99)
unsigned char	gRTC_control;			// qsb Rg[̃f[^
unsigned char	gRTC_status;			// qsb XeC^X̃f[^
unsigned char	gCount_sqw;				// rpvM̃JEg
unsigned int	gTemperature;			// qsb xf[^iPOO{̒l 3075 : 30.75xj

unsigned char	rtc_1s_flg;				// qsbĂ鎞̂Pboߗp

//***************************
// RTC Initial
//  qsb̏ݒ
//***************************
void	rtc_init(void)
{
	unsigned char	t;

	spi_init();

	// Pbo͐ݒ
	//  PD2(INT0): Pulse In from RTC Square Wave Signal(1Hz,4kHz,8kHz,32kHz)
	//  PD3(INT1): Pulse In from RTC Square Wave Signal(1Hz,4kHz,8kHz,32kHz)

	// INT0 Port Initial
//	DDRD &= ~(1 << DDD2);		// Port-D Bit2 : Intput
//	PORTD |= (1 << PD2);		// PD2 Pullup

	// INT1 Port Initial
	DDRD &= ~(1 << DDD3);		// Port-D Bit3 : Intput
	PORTD |= (1 << PD3);		// PD3 Pullup

	rtc_status_read();			// RTC XeC^Xf[^ǂݏo
	rtc_control_read();			// RTC Rg[f[^ǂݏo

	t = gRTC_control;			// ǂݏoRg[f[^ꎞޔ
	gRTC_control = 0x00;		// EOSC:0, BBSQW:0, CONV:0, RS2:0 RS1:0(1Hz) INTCN:0(SQUARE-WAVE OUTPUT), A2IE:0, A1IE=0
	rtc_control_write();		// RTC Rg[f[^
	gRTC_control = t;			// ꎞޔf[^߂

	// INT0 Interrupt Initial
//	EICRA |=  (1 << ISC01);		// INT0 Falling Edge Interrupt
//	EICRA &= ~(1 << ISC00);		// INT0 Falling Edge Interrupt
//	EIMSK |= (1 << INT0);		// INT0 Interrupt Enable

	// INT1 Interrupt Initial
	EICRA |=  (1 << ISC11);		// INT1 Falling Edge Interrupt
	EICRA &= ~(1 << ISC10);		// INT1 Falling Edge Interrupt
	EIMSK |= (1 << INT1);		// INT1 Interrupt Enable

}

//***************************
// RTC ǂݏof[^NA
//***************************
void	rtc_read_data_clear(void)
{
	gRTC_sec = 0;			// qsb b̃f[^ (BCD 0--59)
	gRTC_min = 0;			// qsb ̃f[^ (BCD 0--59)
	gRTC_hou = 0;			// qsb ̃f[^ (BCD 0--23j
	gRTC_wday = 0;			// qsb j̃f[^ (BCD 1--7)
	gRTC_day = 0;			// qsb ̃f[^ (BCD 1--31)
	gRTC_mon = 0;			// qsb ̃f[^ (BCD 1--12j
	gRTC_year = 0;			// qsb Ñf[^ (BCD 0--99)
}

//***************************
// RTC ǂݏo
//  ǂݏof[^͈ȉ̕ϐɓ
//	unsigned char	gRTC_sec;	// qsb b̃f[^ (BCD 0--59)
//	unsigned char	gRTC_min;	// qsb ̃f[^ (BCD 0--59)
//	unsigned char	gRTC_hou;	// qsb ̃f[^ (BCD 0--23j
//	unsigned char	gRTC_day;	// qsb j̃f[^ (BCD 1--7)
//	unsigned char	gRTC_date;	// qsb ̃f[^ (BCD 1--31)
//	unsigned char	gRTC_mon;	// qsb ̃f[^ (BCD 1--12j
//	unsigned char	gRTC_year;	// qsb Ñf[^ (BCD 0--99)
//***************************
void	rtc_read(void)
{
	unsigned char	data;
	unsigned char	dumy;


	spi_start();
	dumy = spi_rd_wr(0x00);		// SPI  AhX

	data = spi_rd_wr(dumy);		// SPI ǂݏo
	gRTC_sec = data;			// qsb b̃f[^ (BCD 0--59)

	data = spi_rd_wr(dumy);		// SPI ǂݏo
	gRTC_min = data;			// qsb ̃f[^ (BCD 0--59)

	data = spi_rd_wr(dumy);		// SPI ǂݏo
	gRTC_hou = data;			// qsb ̃f[^ (BCD 0--23j

	data = spi_rd_wr(dumy);		// SPI ǂݏo
	gRTC_wday = data;			// qsb j̃f[^ (BCD 1--7)

	data = spi_rd_wr(dumy);		// SPI ǂݏo
	gRTC_day = data;			// qsb ̃f[^ (BCD 1--31)

	data = spi_rd_wr(dumy);		// SPI ǂݏo
	gRTC_mon = data;			// qsb ̃f[^ (BCD 1--12j

	data = spi_rd_wr(dumy);		// SPI ǂݏo
	gRTC_year = data;			// qsb Ñf[^ (BCD 0--99)

	spi_stop();

}

//***************************
// RTC 
//  ݃f[^͈ȉ̕ϐɓĂ镨Ƃ
//	unsigned char	gRTC_sec;	// qsb b̃f[^ (BCD 0--59)
//	unsigned char	gRTC_min;	// qsb ̃f[^ (BCD 0--59)
//	unsigned char	gRTC_hou;	// qsb ̃f[^ (BCD 0--23j
//	unsigned char	gRTC_wday;	// qsb j̃f[^ (BCD 1--7)
//	unsigned char	gRTC_day;	// qsb ̃f[^ (BCD 1--31)
//	unsigned char	gRTC_mon;	// qsb ̃f[^ (BCD 1--12j
//	unsigned char	gRTC_year;	// qsb Ñf[^ (BCD 0--99)
//***************************
void	rtc_write(void)
{
	unsigned char	dumy;

	gRTC_sec &= 0x7F;
	spi_start();						// SPI Jn

	dumy=spi_rd_wr(0x00 | 0x80);		// SPI  AhX

	dumy=spi_rd_wr(gRTC_sec);			// b̃f[^ (BCD 0--59)
	dumy=spi_rd_wr(gRTC_min);			// ̃f[^ (BCD 0--59)
	dumy=spi_rd_wr(gRTC_hou);			// ̃f[^ (BCD 0--23j
	dumy=spi_rd_wr(gRTC_wday);			// j̃f[^ (BCD 1--7)
	dumy=spi_rd_wr(gRTC_day);			// ̃f[^ (BCD 1--31)
	dumy=spi_rd_wr(gRTC_mon);			// ̃f[^ (BCD 1--12j
	dumy=spi_rd_wr(gRTC_year);			// Ñf[^ (BCD 0--99)

	spi_stop();

}

//***************************
// RTC Rg[f[^ǂݏo
//  ǂݏof[^ gRTC_control ɓ
//
//
//***************************
void	rtc_control_read(void)
{
	unsigned char	data;
	unsigned char	dumy;

	spi_start();									// SPI Jn

	dumy = spi_rd_wr(RTC_CONTROL_ADDR);				// SPI  AhX

	data = spi_rd_wr(dumy);							// SPI ǂݏo
	gRTC_control = data;							// qsb Rg[̃f[^

	spi_stop();

}
//***************************
// RTC Rg[f[^
//  ݃f[^ unsigned char gRTC_control ɓĂ镨Ƃ
//***************************
void	rtc_control_write(void)
{
	unsigned char	dumy;

	spi_start();									// SPI Jn

	dumy = spi_rd_wr(RTC_CONTROL_ADDR | 0x80);			// SPI  AhX
	dumy = spi_rd_wr(gRTC_control);					// Rg[f[^

	spi_stop();

}

//***************************
// RTC XeC^Xf[^ǂݏo
//  ǂݏof[^ gRTC_status ɓ
//
//
//***************************
void	rtc_status_read(void)
{
	unsigned char	data=0;
	unsigned char	dumy;

	spi_start();									// SPI Jn

	dumy = spi_rd_wr(RTC_SATUS_ADDR);				// SPI  AhX

	data = spi_rd_wr(dumy);							// SPI ǂݏo Ō
	gRTC_status = data;								// qsb XeC^X̃f[^

	spi_stop();

}
//***************************
// RTC XeC^Xf[^
//  ݃f[^ unsigned char gRTC_status ɓĂ镨Ƃ
//***************************
void	rtc_status_write(void)
{
	unsigned char	dumy;

	spi_start();									// SPI Jn

	dumy = spi_rd_wr(RTC_SATUS_ADDR | 0x80);		// SPI  AhX
	dumy = spi_rd_wr(gRTC_status);					// XeC^Xf[^

	spi_stop();

}
//*****************************************
// xANCWJn
//*****************************************
void	temperature_start(void)
{
	gRTC_control |= RTC_CONTROL_CONV;	// CONV:1
	rtc_control_write();				// RTC Rg[f[^

}
//*****************************************
// xǂݏo
//@ǂݏof[^
//   gTemperature_upgTemperature_dw ɓ
//  ANCWIĂȂ́@Ȃ
//*****************************************
void	temperature_read(void)
{
	unsigned char	data_u;
	unsigned char	data_l;
	unsigned char	dumy;

	rtc_control_read();								// RTC Rg[f[^ǂݏo

	if(	(gRTC_control & RTC_CONTROL_CONV) != 0){	// CONV==1 ANCWIĂȂ
		return;										// xǂ܂ȂŏI
	}

	// xǂݍ
	spi_start();									// SPI Jn

	dumy = spi_rd_wr(RTC_TEMPE_UP_ADDR);			// SPI ݃AhXixʃf[^j
	data_u = spi_rd_wr(dumy);						// SPI ǂݏo
	data_l = spi_rd_wr(dumy);						// SPI ǂݏo

	spi_stop();

	gTemperature = (data_u & ~0x80);
	gTemperature *= 100;
	if((data_l & 0x80)!=0){
		gTemperature += 50;
	}
	if((data_l & 0x40)!=0){
		gTemperature += 25;
	}

}
//*****************************************
// INT1 Interrupt
//  Start Bit Falling Edge Sence
// 	qsb@rpvM
//  \߁@qsb̏ݒłPbo͂ɐݒ肷邱
//*****************************************
ISR (INT1_vect)
{
	gCount_sqw++;								// rpvM̃JEg@{P
	rtc_1s_flg++;								// qsbĂ鎞̂Pboߗp

	if(tim_1s_no_key_timeup != 0){				// L[͖^CvAbvpPbPʃ^C}[
		tim_1s_no_key_timeup--;
	}
}
